import { system,ItemStack,world,EquipmentSlot,EntityComponentTypes, GameMode } from "@minecraft/server";
import { RawtextBuilder, showGuidePage, showPage } from "./GuideBook";
import { ModalFormData, ActionFormData, MessageFormData } from "@minecraft/server-ui";

export const VERSION = "1.1.0";
export const VERSION_HISTORY = ["1.1.0"];

system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    const inventory = player.getComponent(EntityComponentTypes.Inventory);

    if (inventory === undefined) return;
    const container = inventory.container;
    if (container === undefined) return;

    // Update inventory items with the typeId "sg_settlements:guide"
    for (let i = 0; i < container.size; i++) {
      const itemStack = container.getItem(i);
      if (itemStack === undefined) continue;
      if (itemStack.typeId === "sg_settlements:guide") {
        const lore = itemStack.getLore() || []; // Get the existing lore or start with an empty array
        if (!lore.includes("§r§7Bedrock Essential")) {
          lore.push("§r§7Bedrock Essential");
          itemStack.setLore(lore);
          container.setItem(i, itemStack);
        }
      }
    }
  }
});








world.afterEvents.playerSpawn.subscribe((event) => {
  const { initialSpawn, player } = event;
  if (!initialSpawn) return;

  const version = player.getDynamicProperty("sg_settlements_version");
  if (version === undefined) {
    player.setDynamicProperty("sg_settlements_version", VERSION);
    if (player.hasTag("sg_settlements_received_guide")) {
      player.removeTag("sg_settlements_received_guide");
      player.sendMessage({
        translate: "sg_settlements.announce_update",
        with: [VERSION],
      });
      return;
    }
    player.dimension.spawnItem(
      new ItemStack("sg_settlements:guide"),
      player.location
    );
  } else if (version !== VERSION) {
    player.setDynamicProperty("sg_settlements_version", VERSION);
    player.sendMessage({
      translate: "sg_settlements.announce_update",
      with: [VERSION],
    });
  }
});

function roundCoordinates(value) {
  return Math.floor(value);
}

function showSettlement(player) {
  const settlements = world.scoreboard.getObjectives();
  const settlement = settlements.find((obj) => {
    if (obj.id.startsWith("sg_settlement:")) {
      // Ensure it starts with 'sg_settlement:'
      const settlementDetails = parseSettlementObjective(obj.id); // Parse the settlement details
      return settlementDetails && settlementDetails.owner === player.name; // Check if the player is the owner
    }
    return false; // If it doesn't start with 'sg_settlement:', skip it
  });



  if (!settlement) {
    // If the player doesn't own a settlement
    const form = new ActionFormData();
    form.title("No Settlement Found");
    form.body("You do not have a settlement. Go to the location where you'd like to create one and press the button below.");
    form.button("Create Settlement", "textures/ui/color_plus.png");

    form.show(player).then((res) => {
      if (res.canceled) return;

      // Prompt the player to create a new settlement
      handleCreateSettlementForm(player);
    });
  } else {
    const parts = settlement.id.split(";"); 
    const name = parts[0].substring(`sg_settlement:`.length);

    const form = new ActionFormData();
    form.title(`§l§8${name}`);
    form.body(`§7Manage your settlement\n§f"${name}"§7:`)
    form.button("Allowed Players", "textures/sg/settlements/icons/players.png");
    form.button("Settlement Options", "textures/sg/settlements/icons/options.png");
    form.button("Change Name", "textures/sg/settlements/icons/changename.png");
    form.button("Test Radius", "textures/ui/equipped_item_border.png");
    form.button("Delete Settlement", "textures/ui/trash.png");
    form.button("§oGo Back");

    form.show(player).then((res) => {
      if (res.canceled) return;

      switch (res.selection) {
        case 0:
          showAllowedPlayers(player, name);
          break;
        case 1:
          showSettlementOptions(player, name);
          break;
        case 2:
          changeSettlementName(player, name);
          break;
        case 3:
          testRadius(player, name);
          break;
        case 4:
          confirmDeleteSettlement(player, name);
          break;
        case 5:
          showMainPage(player);
          break;
      }
    });
  }
}

function testRadius(player, name, particleCount = 125) {
  // Fetch all objectives
  const objectives = world.scoreboard.getObjectives();
  const objective = objectives.find((obj) =>
    obj.id.startsWith(`sg_settlement:${name}`)
  );

  if (!objective) {
    player.sendMessage(`§cSettlement ${name} not found.`);
    return;
  }

  // Extract the settlement data
  const [_, ownerName, allowedPlayersStr, optionStatesStr, coordinatesStr] =
    objective.id.split(";");

  // Parse option states and coordinates
  const optionStates = optionStatesStr.split(",");
  const radius = parseInt(optionStates[1], 10); // Extract radius
  const [x, y, z] = coordinatesStr.split(",").map(Number);
  const coordinates = { x, y, z };

  // Function to spawn a particle
  particleCount = radius * 5.5
  function spawnParticle(particleType, location) {
    player.dimension.spawnParticle(particleType, location);
  }
  function spawnRing(coordinates, heightOffset) {
    // Calculate and spawn particles around the radius
    const angleIncrement = (2 * Math.PI) / particleCount; // Angle between particles

    for (let i = 0; i < particleCount; i++) {
      const angle = i * angleIncrement;
      const particleX = coordinates.x + radius * Math.cos(angle);
      const particleZ = coordinates.z + radius * Math.sin(angle);
      const particleY = coordinates.y + heightOffset;

      const particleLocation = { x: particleX, y: particleY, z: particleZ };

      spawnParticle("sg:radius_test", particleLocation);
    }
  }

  system.runTimeout(() => {spawnRing(coordinates,0)},0)
  system.runTimeout(() => {spawnRing(coordinates,6)},0)
  system.runTimeout(() => {spawnRing(coordinates,12)},0)

  player.sendMessage(`§aParticles have been spawned around the radius of settlement ${name}.`);
}


function parseSettlementObjective(objectiveId) {
  // Split the ID by ";"
  const parts = objectiveId.split(";");

  // Extract each part of the settlement details
  const [prefixedName, owner, allowedPlayers, optionStates, coordinates] = parts;

  // Remove the "sg_settlement:" prefix to get just the settlement name
  const name = prefixedName.replace("sg_settlement:", "");

  // Split the allowed players into an array
  const allowedPlayersArray = allowedPlayers ? allowedPlayers.split(",") : [];

  const [forceField, radius, pvp] = optionStates.split(",");

  // Split coordinates and convert to numbers
  const [x, y, z] = coordinates.split(",").map(Number);

  // Return all relevant parameters in a structured format
  return {
    name,                        // Settlement name
    owner,                       // Owner of the settlement
    allowedPlayers: allowedPlayersArray,  // Array of allowed players
    options: {                   // Object for the option states
      forceField,
      radius,
      pvp
    },
    coordinates: {               // Coordinates as an object
      x, y, z
    }
  };
}

function checkPlayerLocation() {
  world.getPlayers().forEach((player) => {
    const settlements = world.scoreboard.getObjectives().filter((obj) => obj.id.startsWith("sg_settlement:"));
    let isInSettlement = false;

    settlements.forEach((settlement) => {
      const settlementDetails = parseSettlementObjective(settlement.id);
      const { name, coordinates } = settlementDetails;
      const { x, y, z } = coordinates;
      const dist = Math.sqrt((player.location.x - x) ** 2 + (player.location.z - z) ** 2);
      const radius = parseInt(settlementDetails.options.radius);

      if (dist <= radius) {
        isInSettlement = true;
        if (!player.hasTag(`sg_settlement:${name}`)) {
          enterSettlement(player, name, settlement.id);
        }
        
        // Parse the objective data
        const settlementDetails = parseSettlementObjective(settlement.id);
        const { allowedPlayers, options, owner } = settlementDetails;
        if (options.forceField == "1" && !(allowedPlayers.includes(player.name) || owner == player.name)) {
          // Calculate direction and strength for knockback
          const directionX = player.location.x - x;
          const directionZ = player.location.z - z;
          // Apply knockback
          player.applyKnockback(directionX, directionZ, 2, 0.1);
          player.playSound("note.bit",{volume:1,pitch:0.5,location:player.location})
          player.playSound("note.bass",{volume:1,pitch:1,location:player.location})
        }
        if (options.pvp) {
          player.addEffect("weakness",20,{amplifier: 255, showParticles: false})
        }
      } else if (!isInSettlement && player.hasTag(`sg_settlement:${name}`)) {
        exitSettlement(player);
      }
    });
  });
}

system.runInterval(checkPlayerLocation, 10); // Check every 10 ticks

function enterSettlement(player, name, id) {
  // Retrieve the specific objective by id
  const objective = world.scoreboard.getObjective(id);
  if (!objective) {
    player.sendMessage(`§cSettlement ${name} does not exist.`);
    return;
  }

  // Parse the objective data
  const settlementDetails = parseSettlementObjective(objective.id);
  const { allowedPlayers, options, owner } = settlementDetails;

  // Check if the player is allowed or has a tag for this settlement
  if (!player.hasTag(`sg_settlement:${name}`)) {
    if (allowedPlayers.includes(player.name) || owner == player.name) {
      player.onScreenDisplay.setTitle(`§a${name}`);
      player.onScreenDisplay.updateSubtitle(`§e${welcomeSettlement[Math.floor(Math.random() * welcomeSettlement.length)]}`);
    } else {
      player.setGameMode(GameMode.adventure);
      player.onScreenDisplay.setTitle(`§c${name}`);
      player.onScreenDisplay.updateSubtitle(`§e${notWelcomeSettlement[Math.floor(Math.random() * notWelcomeSettlement.length)]}`);
    }
    player.addTag(`sg_settlement:${name}`);
  }
}

function exitSettlement(player) {
  // Retrieve all scoreboard objectives
  const objectives = world.scoreboard.getObjectives();
  
  // Find the settlement tag in the player's tags
  const settlementTag = player.getTags().find(tag => tag.startsWith("sg_settlement:"));
  
  if (!settlementTag) {
    player.sendMessage("§cYou are not in any settlement.");
    return;
  }

  // Find the corresponding objective
  const objective = objectives.find(obj => obj.id.startsWith(settlementTag));
  
  if (objective) {
    const settlementDetails = parseSettlementObjective(objective.id);
    removeSettlementTags(player)
    player.setGameMode(GameMode.survival);

    player.onScreenDisplay.setTitle("§2Wilderness");
    player.onScreenDisplay.updateSubtitle(`§e${exitMessages[Math.floor(Math.random() * exitMessages.length)].replace("${name}", settlementDetails.name)}`);
  } else {
    player.sendMessage("§cSettlement data could not be found.");
  }
}

function removeSettlementTags(player) {
  const settlementTags = player.getTags().filter((tag) => tag.startsWith("sg_settlement:"));

  if (settlementTags.length > 0) {
    settlementTags.forEach((tag) => {
      player.removeTag(tag);
    });
  }
}






function showAllowedPlayers(player, settlementName) {
  const objectives = world.scoreboard.getObjectives();
  
  // Find the specific settlement objective
  const settlementObjective = objectives.find((obj) =>
    obj.id.startsWith(`sg_settlement:${settlementName}`)
  );

  if (!settlementObjective) {
    player.sendMessage(`§cSettlement ${settlementName} not found.`);
    return;
  }

  // Parse the settlement objective data
  const settlementDetails = parseSettlementObjective(settlementObjective.id);
  const allowedPlayers = settlementDetails.allowedPlayers;

  // Create the form to display allowed players
  const form = new ActionFormData().title(`${settlementName} Allowed Players`);

  if (allowedPlayers.length === 0) {
    form.body("§7No players are currently allowed in this settlement.");
  } else {
    allowedPlayers.forEach((allowedPlayer) => {
      const online = world.getPlayers().some((p) => p.name === allowedPlayer);
      form.button(
        `${allowedPlayer} - ${online ? "§aOnline" : "§cOffline"}`,
        `${online ? "textures/sg/settlements/icons/allowed.png" : "textures/sg/settlements/icons/offline.png"}`
      );
    });
  }

  // Add option to add/remove players if the player is the owner
  if (settlementDetails.owner === player.name) {
    form.button("Add Player", "textures/sg/settlements/icons/add.png");
    if (allowedPlayers.length > 0) {
      form.button("Remove Player", "textures/sg/settlements/icons/remove.png");
    }
  }

  form.button("§oGo Back");

  form.show(player).then((res) => {
    if (res.canceled || res.selection === undefined) return;

    const goBackIndex = allowedPlayers.length + (settlementDetails.owner === player.name ? 2 : 0);

    if (res.selection === goBackIndex) {
      showSettlement(player);
    } else if (settlementDetails.owner === player.name) {
      if (res.selection === allowedPlayers.length) {
        // Add a player to the allowed list
        addAllowedPlayerForm(player, settlementName);
      } else if (allowedPlayers.length > 0 && res.selection === allowedPlayers.length + 1) {
        // Remove a player from the allowed list
        removeAllowedPlayerForm(player, settlementName);
      } else if (res.selection < allowedPlayers.length) {
        // Handle any action related to the selected allowed player
        showAllowedPlayers(player, settlementName); // Refresh the list
      }
    } else if (res.selection < allowedPlayers.length) {
      // Refresh the list for other allowed players
      showAllowedPlayers(player, settlementName);
    }
  });
}

// Function to add allowed players
function addAllowedPlayerForm(player, settlementName) {
  const availablePlayers = world.getPlayers().filter((p) => !p.hasTag(settlementName));
  const form = new ModalFormData()
    .title("Add Allowed Player")
    .dropdown("Select Player", availablePlayers.map((p) => p.name))
    .submitButton("Add Player");

  form.show(player).then((res) => {
    if (res.canceled) return;

    const selectedPlayerIndex = res.formValues[0];
    const playerName = availablePlayers[selectedPlayerIndex].name;
    
    // Get the existing settlement objective
    const objectives = world.scoreboard.getObjectives();
    const objective = objectives.find((obj) =>
      obj.id.startsWith(`sg_settlement:${settlementName}`)
    );
    if (!objective) {
      player.sendMessage(`§cSettlement ${settlementName} not found.`);
      return;
    }

    // Extract the settlement details from the objective ID
    const [_, ownerName, allowedPlayersStr, optionStatesStr, coordinatesStr] = objective.id.split(";");
    
    // Update the allowed players list
    const allowedPlayers = allowedPlayersStr ? allowedPlayersStr.split(",") : [];
    if (!allowedPlayers.includes(playerName)) {
      allowedPlayers.push(playerName);
    }

    // Parse the option states and coordinates
    const optionStates = optionStatesStr.split(",");
    const [x, y, z] = coordinatesStr.split(",").map(Number);
    const coordinates = { x, y, z };

    // Re-create the settlement with the updated allowed players list
    createSettlement(player, settlementName, ownerName, allowedPlayers, optionStates, coordinates);

    player.sendMessage(`§aYou have added ${playerName} to the allowed players of ${settlementName}.`);
    removeSettlementTags(world.getPlayers({name: playerName}))
  });
}


function removeAllowedPlayerForm(player, settlementName) {
  const objectives = world.scoreboard.getObjectives();
  const objective = objectives.find((obj) =>
    obj.id.startsWith(`sg_settlement:${settlementName}`)
  );
  if (!objective) {
    player.sendMessage(`§cSettlement ${settlementName} not found.`);
    return;
  }

  // Extract the settlement data
  const [_, ownerName, allowedPlayersStr, optionStatesStr, coordinatesStr] = objective.id.split(";");
  
  // Parse the allowed players list
  const allowedPlayers = allowedPlayersStr ? allowedPlayersStr.split(",") : [];
  
  if (allowedPlayers.length === 0) {
    player.sendMessage(`§cThere are no allowed players to remove.`);
    return;
  }

  // Create a form to select which allowed player to remove
  const form = new ModalFormData()
    .title("Remove Allowed Player")
    .dropdown("Select Player", allowedPlayers)
    .submitButton("Remove Player");

  form.show(player).then((res) => {
    if (res.canceled) return;

    const selectedPlayerIndex = res.formValues[0];
    const playerName = allowedPlayers[selectedPlayerIndex];
    
    // Remove the player from the allowed list
    const updatedAllowedPlayers = allowedPlayers.filter((p) => p !== playerName);
    
    // Parse the option states and coordinates
    const optionStates = optionStatesStr.split(",");
    const [x, y, z] = coordinatesStr.split(",").map(Number);
    const coordinates = { x, y, z };

    // Re-create the settlement with the updated allowed players list
    createSettlement(player, settlementName, ownerName, updatedAllowedPlayers, optionStates, coordinates);

    player.sendMessage(`§aYou have removed ${playerName} from the allowed players of ${settlementName}.`);
    removeSettlementTags(world.getPlayers({ name: playerName }));
  });
}

// Function to generate the settlement objective ID with all parameters
function createSettlementObjectiveID(settlementName, ownerName, allowedPlayers, optionStates, coordinates) {
  // Format: "sg_settlement:<settlementName>;<ownerName>;<allowedPlayers>;<optionStates>;<coordinates>"
  const allowedPlayersString = allowedPlayers.join(",");
  const optionsString = optionStates.join(",");
  const coordinatesString = `${coordinates.x},${coordinates.y},${coordinates.z}`;
  return `sg_settlement:${settlementName};${ownerName};${allowedPlayersString};${optionsString};${coordinatesString}`;
}

// General function to create or update a settlement
function createSettlement(player, settlementName, ownerName, allowedPlayers = [], optionStates = [0, "32", 0], coordinates) {
  if (!settlementName || !ownerName || !coordinates) {
    player.sendMessage("§cSettlement parameters are missing.");
    return;
  }

  // Generate the scoreboard objective data
  const settlementData = createSettlementObjectiveID(settlementName, ownerName, allowedPlayers, optionStates, coordinates);
  
  // Remove any existing objective with the same name
  const objectives = world.scoreboard.getObjectives();
  const existingObjective = objectives.find((obj) =>
    obj.id.startsWith(`sg_settlement:${settlementName}`)
  );
  if (existingObjective) {
    world.scoreboard.removeObjective(existingObjective);
  }

  // Add the new settlement objective to the scoreboard
  world.scoreboard.addObjective(settlementData);
}

// Example usage of the createSettlement function
function handleCreateSettlementForm(player) {
  const form = new ModalFormData();
  form.title("Create a Settlement");
  form.textField("Enter Settlement Name", "My Settlement");
  form.slider("Settlement Radius", 8, 32, 1, 16);

  form.show(player).then((res) => {
    if (res.canceled) return;

    const settlementName = res.formValues[0];
    const radius = res.formValues[1];

    if (!settlementName) {
      player.sendMessage("§cPlease provide a name for your settlement.");
      return;
    }

    const x = Math.floor(player.location.x);
    const y = Math.floor(player.location.y);
    const z = Math.floor(player.location.z);

    const coordinates = { x, y, z };
    const ownerName = player.name;
    const allowedPlayers = []; // Start with an empty allowed players list
    const optionStates = [0, radius.toString(), 0]; // Example options (first one is a boolean for protection)

    // Create the settlement using the provided details
    createSettlement(player, settlementName, ownerName, allowedPlayers, optionStates, coordinates);
  });
}


function showSettlementOptions(player, settlementName) {
  // Fetch all objectives
  const objectives = world.scoreboard.getObjectives();
  const objective = objectives.find((obj) =>
    obj.id.startsWith(`sg_settlement:${settlementName}`)
  );

  if (!objective) {
    player.sendMessage(`§cSettlement ${settlementName} not found.`);
    return;
  }

  // Extract settlement data from objective ID
  const [_, ownerName, allowedPlayersStr, optionStatesStr, coordinatesStr] =
    objective.id.split(";");

  // Parse option states
  const optionStates = optionStatesStr.split(",");
  const forceFieldEnabled = optionStates[0] === "1"; // Convert to boolean
  const radius = parseInt(optionStates[1], 10); // Convert to number
  const pvp = optionStates[2] === "1"; // Convert to boolean

  // Parse coordinates
  const [x, y, z] = coordinatesStr.split(",").map(Number);
  const coordinates = { x, y, z };

  // Create a form for settlement options
  const form = new ModalFormData();
  form.title(`${settlementName} Options`);
  form.toggle("Force Field Enabled", forceFieldEnabled);
  form.slider("Radius", 8, 32, 1, radius);
  form.toggle("Allow PVP", pvp);
  form.submitButton("Save Options")

  form
    .show(player)
    .then((res) => {
      if (res.canceled) return;

      // Collect new option states
      const updatedForceFieldEnabled = res.formValues[0] ? "1" : "0"; // Convert boolean to "1" or "0"
      const updatedRadius = res.formValues[1]; // Slider value (number)
      const updatedPvp = res.formValues[2] ? "1" : "0"; // Convert boolean to "1" or "0"

      // Prepare updated option states
      const updatedOptionStates = `${updatedForceFieldEnabled},${updatedRadius},${updatedPvp}`;

      // Ensure createSettlement function exists and is called correctly
      if (typeof createSettlement === "function") {
        createSettlement(
          player,
          settlementName,
          ownerName,
          allowedPlayersStr ? allowedPlayersStr.split(",") : [], // Parse allowed players
          updatedOptionStates.split(","), // Ensure optionStates is an array
          coordinates // Pass the coordinates object
        );
        player.sendMessage(
          `§aYou have updated the options for ${settlementName}.`
        );
        showSettlement(player)
      } else {
        player.sendMessage(
          "§cError: createSettlement function is not defined."
        );
      }
    })
    .catch((error) => {
      player.sendMessage(`§cAn error occurred: ${error.message}`);
      console.error(error);
    });
}





function changeSettlementName(player, oldName) {
  const form = new ModalFormData();
  form.textField("New Settlement Name", oldName);
  form.submitButton("Save Name")

  form.show(player).then((res) => {
    if (res.canceled) return;
    const newName = res.formValues[0];
    if (!newName || newName === oldName) {
      player.sendMessage("§cInvalid or same name provided.");
      return;
    }

    // Fetch all objectives
    const objectives = world.scoreboard.getObjectives();
    const oldObjective = objectives.find(obj => obj.id.startsWith(`sg_settlement:${oldName}`));

    if (!oldObjective) {
      player.sendMessage("§cSettlement not found.");
      return;
    }

    // Parse old settlement details
    const oldSettlementDetails = parseSettlementObjective(oldObjective.id);
    const { owner, allowedPlayers, options, coordinates } = oldSettlementDetails;

    // Create new objective data with the updated name
    const newObjectiveId = `sg_settlement:${newName};${owner};${allowedPlayers};${options.forceField},${options.radius},${options.pvp};${coordinates.x},${coordinates.y},${coordinates.z}`;

    // Remove the old objective and add the new one
    world.scoreboard.removeObjective(oldObjective.id);
    world.scoreboard.addObjective(newObjectiveId);

    // Retrieve all players with the old settlement tag
    const oldTag = `sg_settlement:${oldName}`;
    const playersWithOldTag = world.getPlayers().filter(p => p.getTags().includes(oldTag));

    // Update player tags
    playersWithOldTag.forEach(p => {
      removeSettlementTags(p); // Remove all settlement tags for this player
    });
    showSettlement(player);

    player.sendMessage(`§aSettlement name changed from ${oldName} to ${newName}.`);
  });
}



function confirmDeleteSettlement(player, settlementName) {
  const form = new ActionFormData();
  form.title(`Delete ${settlementName}?`);
  form.body("Are you sure you want to delete this settlement?");
  form.button("Yes", "textures/ui/trash.png");
  form.button("Cancel", "textures/ui/cancel.png");

  form.show(player).then((res) => {
    if (res.canceled || res.selection === undefined) return;

    if (res.selection === 0) {
      // Fetch all objectives
      const objectives = world.scoreboard.getObjectives();
      const objectiveToRemove = objectives.find(obj => obj.id.startsWith(`sg_settlement:${settlementName}`));

      if (objectiveToRemove) {
        // Remove the objective from the scoreboard
        world.scoreboard.removeObjective(objectiveToRemove.id);

        // Remove settlement tags from all players
        const oldTag = `sg_settlement:${settlementName}`;
        const playersWithOldTag = world.getPlayers().filter(p => p.getTags().includes(oldTag));

        playersWithOldTag.forEach(p => {
          removeSettlementTags(p); // Remove all settlement tags for this player
        });

        // Notify the player
        player.sendMessage(`§cSettlement ${settlementName} deleted.`);
      } else {
        player.sendMessage("§cSettlement not found.");
      }
    } else {
      showSettlement(player)
    }
  });
}

const welcomeSettlement = [
  "Welcome home!",
  "Welcome back, honored guest!",
  "You are home again.",
  "Your presence is\nalways valued here.",
  "Good to see you again\nin the settlement.",
  "This land welcomes its\nrightful visitors.",
  "You belong here, friend.",
  "Enjoy your time in the haven.",
  "Welcome to your safe haven.",
  "You have arrived at your sanctuary.",
  "Your arrival is expected and welcome.",
  "This place is your retreat.",
  "The settlement recognizes you.",
  "Your path leads you home.",
  "You are welcome in these familiar lands.",
  "Welcome to your home away from home.",
  "A warm welcome to familiar lands.",
  "Good to see you back in the settlement!",
  "You belong here. Welcome home.",
  "Back to the place\nwhere you are always welcome.",
  "Welcome to your sanctuary.",
  "Home is where the heart is.\nWelcome!",
  "Your presence brings\nlife to the settlement.",
  "Welcome to your piece of paradise.",
  "You've returned to safe grounds.",
  "Welcome back to your land of peace.",
  "This settlement thrives because of you.",
  "Home sweet home! Welcome back!",
  "Good to see you in familiar territory.",
  "Welcome back to your domain.",
  "A place just for you. Welcome home!",
  "Your settlement awaits your presence.",
  "You've returned\nto where you are valued.",
  "Welcome to the land\nthat knows your name.",
  "Safe in your settlement once again.\nWelcome!",
];


const notWelcomeSettlement = [
  "You are not allowed here.",
  "This land is foreign to you.",
  "Your presence is unwelcome.",
  "Trespassing is not permitted here.",
  "This settlement does not recognize you.",
  "You do not have access to this land.",
  "Leave this place immediately.",
  "You are forbidden to step further.",
  "This land is reserved for others.",
  "Strangers are not allowed\nbeyond this point.",
  "You are trespassing in\nrestricted territory.",
  "This settlement is off-limits to you.",
  "Turn back, you are not welcome here.",
  "Your path is blocked here.",
  "This is not a place for outsiders.",
  "Access to this area is denied.",
  "You do not have permission to be here.",
  "This land is protected from outsiders.",
  "You have no claim here.",
  "This territory is not for you.",
  "Turn back, outsider.",
  "This ground is off-limits to you.",
  "You are an intruder on this land.",
  "Only the chosen may enter here.",
  "You are not part of this settlement.",
  "This area is closed to you.",
  "You have no right to enter here.",
  "You shall not pass beyond this point.",
  "Your presence is not permitted.",
  "Keep away from this territory.",
  "These grounds are reserved for others.",
  "You are a stranger in these lands.",
  "Unauthorized access is forbidden.",
  "You are an unwelcome guest.",
  "These borders are closed to you.",
];

const exitMessages = [
  "You have left the safety of\n${name}.",
  "You've stepped away from ${name},\ninto the unknown.",
  "${name} is now behind you,\nthe wild awaits.",
  "The borders of ${name}\nare far behind.",
  "You leave the familiar grounds of\n${name}.",
  "The wilderness surrounds you beyond\n${name}.",
  "${name}'s protection\nno longer shields you.",
  "The wild greets you\nas you exit ${name}.",
  "You depart from ${name},\nheading into the untamed lands.",
  "${name} fades into the distance\nas you enter the wilderness.",
  "The frontier beyond ${name}\nopens before you.",
  "You leave ${name}\nand enter unclaimed territory.",
  "The wildlands stretch before you,\nfar from ${name}.",
  "You wander beyond the reach of\n${name}.",
  "No longer in ${name},\nthe wilderness welcomes you.",
  "The protection of ${name}\nis no longer with you.",
  "You step beyond the borders of ${name}\ninto uncharted territory.",
  "The wilderness stretches before you\nas you leave ${name}.",
  "You've ventured out from ${name}\ninto the open wild.",
  "The safety of ${name} fades\nas you move into the unknown.",
  "You have crossed the boundary of ${name},\nentering the untamed lands.",
  "Beyond ${name}\nlies the wilderness-be cautious.",
  "The wild calls\nas you leave the grounds of ${name}.",
  "You exit ${name},\nheading into lands unclaimed by any.",
  "Leaving ${name} behind,\nthe wilderness now surrounds you.",
];

const BUTTONS = [
  {
    // SETTLEMENT
    translation_key: "sg_settlements.guide.settlement.title",
    icon_path: "textures/sg/settlements/icons/farming.png",
    action: (player) => {
      showSettlement(player);
    },
  },
  {
    // CHANGELOG
    translation_key: "sg_settlements.guide.changelog.title",
    icon_path: "textures/ui/book_addtextpage_default.png",
    action: (player) => {
      const body = new RawtextBuilder();
      body.pushLine({
        translate: "sg_settlements.guide.changelog.body",
        with: [VERSION],
      });
      body.pushLine();
      for (const version of VERSION_HISTORY) {
        body.pushLine({
          translate: "sg_settlements.guide.changelog.subtitle",
          with: [version],
        });
        body.pushLine();
        body.pushLine({
          translate: `sg_settlements.guide.changelog.${version}`,
          with: ["\n"],
        });
        body.pushLine();
        body.pushLine();
      }
      showPage(player, "sg_settlements.guide.changelog.title", body.build(), [
        {
          translation_key: "sg_settlements.guide.common.back_button",
          action: (player) => showMainPage(player),
        },
      ]);
    },
  },
  {
    translation_key: "sg_settlements.guide.common.exit_button",
  },
];

const showMainPage = (player) => {
  showGuidePage(player, "main", 4, BUTTONS);
};

world.afterEvents.itemUse.subscribe((event) => {
  system.run(() => {
    if (event.itemStack.typeId === "sg_settlements:guide") {
      showMainPage(event.source);
    }
  });
});